# ============================================================
# RunAction plugin for Falkon
# Copyright (C) 2018 David Rosca <nowrep@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ============================================================
import Falkon
import os
from PySide6 import QtGui, QtWidgets
from runaction.i18n import i18n


class RunActionButton(Falkon.AbstractButtonInterface):
    def __init__(self, manager):
        super().__init__()
        self.manager = manager

        self.setIcon(QtGui.QIcon(os.path.join(os.path.dirname(__file__), "icon.svg")))
        self.setTitle(i18n("Run Action"))
        self.setToolTip(i18n("Run action on current page"))

        self.clicked.connect(self.onClicked)

    def id(self):
        return "runaction-button"

    def name(self):
        return i18n("RunAction button")

    def onClicked(self, controller):
        self.menu = QtWidgets.QMenu()

        for action in self.manager.getActions(self.webView()):
            self.menu.addAction(action)

        self.menu.addSeparator()
        self.menu.addAction(QtGui.QIcon.fromTheme("configure"), i18n("Configure..."), self.manager.showSettings)

        self.menu.popup(controller.callPopupPosition(self.menu.sizeHint()))
        self.menu.aboutToHide.connect(controller.callPopupClosed)



# ============================================================
# MiddleClickLoader - plugin for Falkon
# Copyright (C) 2018 Juraj Oravec <sgd.orava@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ============================================================

from enum import IntEnum


class MCL_LoadMode(IntEnum):
    NEW_TAB = 0
    CURRENT_TAB = 1
    NEW_WINDOW = 2



# ============================================================
# RunAction plugin for Falkon
# Copyright (C) 2018 David Rosca <nowrep@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ============================================================
import Falkon
import os
import subprocess
from PySide6 import QtCore
from runaction.action import Action
from runaction.settingsdialog import SettingsDialog


class ActionManager(QtCore.QObject):
    actions = []

    def __init__(self, settingsPath, parent=None):
        super().__init__(parent)

        self.settingsPath = settingsPath
        settings = QtCore.QSettings(self.settingsPath + "/extensions.ini", QtCore.QSettings.IniFormat)
        self._disabledActions = settings.value("RunAction/disabledActions") or []
        self.loadActions()

    def getActions(self, webView, r=None):
        out = []
        menus = {}

        for action in list(filter(lambda a: a.id not in self.disabledActions, self.actions)):
            url = webView.url()
            text = ""
            if r and webView.selectedText():
                cond = Action.TypeCondition.Text
                text = webView.selectedText()
            elif r and not r.linkUrl().isEmpty():
                cond = Action.TypeCondition.Link
                url = r.linkUrl()
            elif r and not r.imageUrl().isEmpty():
                cond = Action.TypeCondition.Image
                url = r.imageUrl()
            elif r and not r.mediaUrl().isEmpty():
                cond = Action.TypeCondition.Media
                url = r.mediaUrl()
            else:
                cond = Action.TypeCondition.Page

            if action.testAction(cond, url):
                act = Falkon.Action(action.icon, action.title, self)
                act.triggered.connect(lambda checked, a=action, w=webView, u=url, t=text: self.execAction(a, w, u, t))
                if action.submenu:
                    if action.submenu not in menus:
                        menu = Falkon.Menu(action.menuTitle, webView)
                        menus[action.submenu] = menu
                        out.append(menu)
                    menus[action.submenu].addAction(act)
                else:
                    out.append(act)

        return out

    @property
    def disabledActions(self):
        return self._disabledActions

    @disabledActions.setter
    def disabledActions(self, value):
        settings = QtCore.QSettings(self.settingsPath + "/extensions.ini", QtCore.QSettings.IniFormat)
        settings.setValue("RunAction/disabledActions", value)
        self._disabledActions = value

    def showSettings(self, parent=None):
        dialog = SettingsDialog(self, parent)
        dialog.exec_()

    def execAction(self, action, webView, url, text=""):
        command = action.execAction(url, text)
        if action.actionType == Action.Type.Command:
            subprocess.Popen(command, shell=True)
        elif action.actionType == Action.Type.Url:
            webView.openUrlInNewTab(QtCore.QUrl(command), Falkon.Qz.NT_SelectedTab)

    def loadActions(self):
        self.actions = []

        paths = [
            os.path.join(os.path.dirname(__file__), "actions"),
            os.path.join(self.settingsPath, "runaction")
        ]

        for path in paths:
            if not os.path.exists(path):
                continue
            for file in os.listdir(path):
                if not file.endswith(".desktop"):
                    continue
                fileName = os.path.join(path, file)
                try:
                    action = Action(fileName)
                except Exception as e:
                    print("Failed to parse {}: {}".format(fileName, e))
                finally:
                    if action.supported:
                        self.actions.append(action)



# ============================================================
# RunAction plugin for Falkon
# Copyright (C) 2018 David Rosca <nowrep@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ============================================================
import Falkon
import os
import re
import enum
import shlex
from PySide6 import QtCore, QtGui


class Action():
    class Type(enum.Enum):
        Invalid, Url, Command = range(3)

    class TypeCondition(enum.Enum):
        Page, Link, Image, Media, Text = range(5)

    id = ""
    title = ""
    menuTitle = ""
    icon = QtGui.QIcon()
    actionType = Type.Invalid
    typeCondition = [ TypeCondition.Page, TypeCondition.Link, TypeCondition.Image, TypeCondition.Media ]
    urlCondition = ".*"
    submenu = ""
    normalExec = ""
    textExec = ""
    supported = False

    def __init__(self, fileName):
        data = Falkon.DesktopFile(fileName)
        self.id = os.path.splitext(os.path.basename(fileName))[0]
        self.title = data.name()
        self.menuTitle = data.comment()
        self.icon = QtGui.QIcon.fromTheme(data.icon(), QtGui.QIcon(os.path.join(os.path.dirname(fileName), data.icon())))
        self.actionType = Action.Type[data.value("X-RunAction-Type")]
        self.typeCondition = list(map(lambda s: Action.TypeCondition[s], data.value("X-RunAction-TypeCondition").split(";")))
        self.urlCondition = data.value("X-RunAction-UrlCondition") or self.urlCondition
        self.submenu = data.value("X-RunAction-Submenu") or self.submenu
        self.normalExec = data.value("X-RunAction-Exec") or self.normalExec
        self.textExec = data.value("X-RunAction-TextExec") or self.normalExec
        self.supported = data.tryExec()

    def testAction(self, condition, url):
        if not self.supported:
            return False
        if condition not in self.typeCondition:
            return False
        if not re.match(self.urlCondition, url.toString()):
            return False
        return True

    def execAction(self, url, text=""):
        url = str(url.toEncoded(), "utf-8")
        if self.actionType == Action.Type.Command:
            url = shlex.quote(url)
            text = shlex.quote(text)
        elif self.actionType == Action.Type.Url:
            url = str(QtCore.QUrl.toPercentEncoding(url), "utf-8")
            text = str(QtCore.QUrl.toPercentEncoding(text), "utf-8")
        command = self.normalExec if text == "" else self.textExec
        command = command.replace("{url}", url)
        command = command.replace("{text}", text)
        command = command.replace("{lang}", QtCore.QLocale.system().name()[:2])
        return command



# ============================================================
# MiddleClickLoader - plugin for Falkon
# Copyright (C) 2018 Juraj Oravec <sgd.orava@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ============================================================

import Falkon
from PySide6 import QtCore, QtGui, QtWidgets

from middleclickloader.mcl_loadmode import MCL_LoadMode
from middleclickloader.mcl_settings import MCL_Settings


class MCL_Handler(QtCore.QObject):
    settingsFile = ""
    loaded = False

    onlyValidUrl = False
    loadMode = 0

    def __init__(self, settingsPath, parent=None):
        super().__init__(parent)

        self.settingsFile = settingsPath + "/extensions.ini"

    def loadSettings(self):
        settings = QtCore.QSettings(self.settingsFile, QtCore.QSettings.IniFormat)
        settings.beginGroup("MiddleClickLoader")
        self.loadMode = int(settings.value("LoadMode", MCL_LoadMode.NEW_TAB))
        self.onlyValidUrl = bool(settings.value("OnlyValidUrl", True))
        settings.endGroup()

        self.loaded = True

    def showSettings(self, parent=None):
        self.settings = MCL_Settings(self.settingsFile, parent)
        self.settings.accepted.connect(self.loadSettings)

        self.settings.exec_()

    def mousePress(self, view, event):
        if not self.loaded:
            self.loadSettings()

        if event.buttons() == QtCore.Qt.MiddleButton:
            res = view.page().hitTestContent(event.pos())

            if res.isContentEditable() or not res.linkUrl().isEmpty():
                return False

            selectionClipboard = QtWidgets.QApplication.clipboard().text(QtGui.QClipboard.Selection)

            if selectionClipboard:
                guessedUrl = QtCore.QUrl.fromUserInput(selectionClipboard)
                isValid = view.isUrlValid(guessedUrl)

                if self.onlyValidUrl and not isValid:
                    return False

                if not isValid:
                    searchManager = Falkon.MainApplication.instance().searchEnginesManager()
                    engine = searchManager.defaultEngine()
                    req = searchManager.searchResult(engine, selectionClipboard)
                    guessedUrl = req.url()

                return self.loadUrl(view, guessedUrl)
        return False

    def loadUrl(self, view, url):
        if self.loadMode == MCL_LoadMode.NEW_TAB:
            view.openUrlInNewTab(url, Falkon.Qz.NT_NotSelectedTab)
        elif self.loadMode == MCL_LoadMode.CURRENT_TAB:
            view.load(url)
        elif self.loadMode == MCL_LoadMode.NEW_WINDOW:
            Falkon.MainApplication.instance().createWindow(Falkon.Qz.BW_NewWindow, url)
        else:
            return False
        return True
